$title  Implan Aggregation Routine -- Aggregates from any dimension dataseta

$ontext

-----------------------------------------------------------------------------------
Thomas F. Rutherford
Department of Economics

version April, 2002

$offtext

$if not exist 'defines\%ds%.set' $abort 'Error:	Cannot find defines\%ds%.set.'
$if not exist 'defines\%ds%.map' $abort 'Error:	Cannot find defines\%ds%.map.'

*	Run the GAMS program which generates source sets:

$call 'gams sets --ds=%ds%'

*       Read the target sets:

$include defines\%ds%.set

*       Read the source sets (these were generated by gtapsets):

set	ss(*)	Sectoral definitions,
	rr(*)	Regional definitions;

$gdxin 'aggr'
$load ss rr

*       Declare the mappings so that we do domain checking on input:

set		mapr(r,rr), maps(s,ss);
parameter	rratio(r,rr), sratio(s,ss);

alias (s,g), (ss,gg);

*       Read the mappings:

$include defines\%ds%.map

*       Assign default mappings if they are not defined:

$if defined maps sratio(s,ss)$maps(s,ss) = 1;
$if defined mapr rratio(r,rr)$mapr(r,rr) = 1;

$if not defined sratio sratio(s,ss)$sameas(s,ss)  = 1;
$if not defined rratio rratio(r,rr)$sameas(r,rr)  = 1;

parameter	sratiochk	Deviations in SRATIO
		rratiochk	Deviations in RRATIO;

sratiochk(ss) = sum(s, sratio(s,ss)) - 1;
rratiochk(rr) = sum(r, rratio(r,rr)) - 1;

abort$smax(ss, abs(sratiochk(ss)) gt 0.01) "Error in SRATIO:",sratiochk,sratio;
abort$smax(rr, abs(rratiochk(rr)) gt 0.01) "Error in RRATIO:",rratiochk,rratio;

sratio(s,ss) = sratio(s,ss) / (1 + sratiochk(ss));
rratio(r,rr) = rratio(r,rr) / (1 + rratiochk(rr));

SET	hh	Private (household) institutions /
		HHL "Households LT5k   (10001)"
		HH5 "Households 5-10k  (10002)"
		HH10 "Households 10-15k (10003)"
		HH15 "Households 15-20k (10004)"
		HH20 "Households 20-30k (10005)"
		HH30 "Households 30-40k (10006)"
		HH40 "Households 40-50k (10007)"
		HH50 "Households 50-70k (10008)"
		HH70 "Households 70k+   (10009)"/,

	trd  Trade partners /
		FTRD "Foreign Trade (25001)"
		DTRD "Domestic Trade (28001)" /,

	pub	Public (government) institutions /
		FND "Federal Government NonDefense (11001)"
		FDF "Federal Government Defense (11002)"
		FIN "Federal Government Investment (11003)"
		SLN "State Local Govt NonEducation (12001)"
		SLE "State Local Govt Education (12002)"
		SIN "State Local Govt Investment (12003)" /;

parameter
	x0(rr,ss,trd)	Exports by market
	m0(rr,gg,trd)	Imports from domestic and foreign sources
	id0(rr,gg,ss)	Intermediate demand
	kd0(rr,ss)	Capital return
	ld0(rr,ss)	Labor demand
	bt0(rr,ss)	Business taxes
	fica(rr)	Labor tax rate (social security)
	i0(rr,gg)	Investment demand by commodity
	g0(rr,gg,pub)	Public demand by commodity
	le0(rr,hh)	Labor endowment
	ke0(rr,hh)	Capital endowment
	cd0(rr,gg,hh)	Consumer demand
	invd0(rr,hh)	Investment;

$log Reading Data...
$if not exist data\%source%.gdx $abort 'Cannot find source data file: data\%source%.gdx'
$gdxin 'data\%source%'
$load x0 m0 id0 kd0 ld0 bt0 fica i0 g0 le0 ke0 cd0 invd0 

*       Aggregate the value arrays:

$log Aggregating Data...

parameter
	x0_(r,s,trd)	Exports by market
	m0_(r,g,trd)	Imports from domestic and foreign sources
	id0_(r,g,s)	Intermediate demand
	kd0_(r,s)	Capital return
	ld0_(r,s)	Labor demand
	bt0_(r,s)	Business taxes
	fica_(r)	Labor tax rate (social security)
	i0_(r,g)	Investment demand by commodity
	g0_(r,g,pub)	Public demand by commodity
	le0_(r,hh)	Labor endowment
	ke0_(r,hh)	Capital endowment
	cd0_(r,g,hh)	Consumer demand
	invd0_(r,hh)	Investment;

x0_(r,s,trd) = sum((rr,ss), rratio(r,rr)*sratio(s,ss)*x0(rr,ss,trd));
m0_(r,s,trd) = sum((rr,ss),rratio(r,rr)*sratio(s,ss)*m0(rr,ss,trd));
id0_(r,g,s) = sum((rr,gg,ss),rratio(r,rr)*sratio(s,ss)*sratio(g,gg)*id0(rr,gg,ss));
kd0_(r,s) = sum((rr,ss),rratio(r,rr)*sratio(s,ss)*kd0(rr,ss));
ld0_(r,s) = sum((rr,ss),rratio(r,rr)*sratio(s,ss)*ld0(rr,ss));
bt0_(r,s) = sum((rr,ss),rratio(r,rr)*sratio(s,ss)*bt0(rr,ss));
fica_(r) = sum(rr, rratio(r,rr)*fica(rr));
i0_(r,g) = sum((rr,gg),rratio(r,rr)*sratio(g,gg)*i0(rr,gg));
g0_(r,g,pub) = sum((rr,gg),rratio(r,rr)*sratio(g,gg)*g0(rr,gg,pub));
le0_(r,hh) = sum(rr,rratio(r,rr)*le0(rr,hh));
ke0_(r,hh) = sum(rr, rratio(r,rr)*ke0(rr,hh));
cd0_(r,g,hh) = sum((rr,gg),rratio(r,rr)*sratio(g,gg)*cd0(rr,gg,hh));
invd0_(r,hh) = sum(rr,rratio(r,rr)*invd0(rr,hh));

execute_unload 'data\%ds%.gdx',r,s,x0_=x0,m0_=m0,id0_=id0,kd0_=kd0,ld0_=ld0,
	bt0_=bt0,fica_=fica,i0_=i0,g0_=g0,le0_=le0,ke0_=ke0,cd0_=cd0,invd0_=invd0;

parameter mktchk;
mktchk(s,trd) = sum(r, x0_(r,s,trd) - m0_(r,s,trd));
display mktchk;
