$TITLE   Diagonalize Production in the IMPLAN SOE Framework

$if not set ds $set ds co

$ontext

This program reads an IMPLAN dataset for a single state and

	(i)  verifies IMPLAN dataset identities
	(ii) interprets the resulting data in the form of a 
	     highly simplified general equilibrium model formulated
	     with MPSGE
	(iii) outputs the dataset to an mrn datafile
	

$offtext

SET	S(*)	Sectors,
	F(*)	Factors,
	TRD(*)	Trade partners,
	I(*)	Institutions;

$GDXIN 'temp.gdx'

$load s f trd i

ALIAS (G,S), (I,II), (TRD,TTRD), (G,GG), (S,SS), (F,FF);

PARAMETER
	MAKE(S,G)	Domestic industry make matrix
	SEXPRT(S,TRD)   Aggregate industry exports
	USE(G,S)        Industry use matrix
	IUSE(G,I)       Institutional use matrix
	FD(F,S)         Industry use of factors
	IMAKE(I,G)      Institutional make matrix
	FS(I,F)         Factor disbursement matrix
	TRNSFER(I,II)   Inter-institutional transfers    

	FEXPRT(F,TRD)   Factor exports
	IEXPRT(I,TRD)   Institutional exports
	SIMPRT(TRD,S)   Industry imports
	FIMPRT(TRD,F)   Factor imports
	IIMPRT(TRD,I)   Institutional imports
	EXPORT(*,G,TRD) Commodity exports
	IMPORT(TRD,G,*) Commodity imports

	TRNSHP(TRD,TTRD) Transhipments;

$load MAKE SEXPRT USE IUSE FD FEXPRT IMAKE FS TRNSFER IEXPRT SIMPRT 
$load FIMPRT IIMPRT TRNSHP EXPORT IMPORT 

*	Input the IMPLAN data:

*.$include data\temp

*	Alternative reference to the set of regions:

alias (g,gg), (s,ss), (i,ii);

SET	HH(I)	Private (household) institutions /
		HHL "Households LT5k   (10001)"
		HH5 "Households 5-10k  (10002)"
		HH10 "Households 10-15k (10003)"
		HH15 "Households 15-20k (10004)"
		HH20 "Households 20-30k (10005)"
		HH30 "Households 30-40k (10006)"
		HH40 "Households 40-50k (10007)"
		HH50 "Households 50-70k (10008)"
		HH70 "Households 70k+   (10009)"/,

	PUB(I)  Public (government) institutions /
		FND "Federal Government NonDefense (11001)"
		FDF "Federal Government Defense (11002)"
		FIN "Federal Government Investment (11003)"
		SLN "State Local Govt NonEducation (12001)"
		SLE "State Local Govt Education (12002)"
		SIN "State Local Govt Investment (12003)" /,

	CGD(I) Investment institutions	/
		ENT Enterprises (Corporations) (13001)
		INV Gross Private Fixed Investment (Capital) (14001)
		STK Inventory Sales Deletions (14002)/,

	FK(F)	Factors allocated to the capital account /
		PROP Proprietary Income (6001)
		OTHP Other Property Income (7001) /;

ALIAS  (trd,ttrd), (g,s), (f,ff), (hh,h_h);


*	Begin by scaling trade flows to match totals:

parameter	aggimport(*,*)	Aggregate imports,
		aggexport(*,*)	Aggregate exports;

aggimport(trd,s) = sum(gg, import(trd,gg,s));
aggimport(trd,i) = sum(gg, import(trd,gg,i));
aggexport(s,trd) = sum(gg, export(s,gg,trd));
aggexport(i,trd) = sum(gg, export(i,gg,trd));

import(trd,g,s)$aggimport(trd,s)
	 = import(trd,g,s) * simprt(trd,s) / aggimport(trd,s);

import(trd,g,i)$aggimport(trd,i)
	 = import(trd,g,i) * iimprt(trd,i) / aggimport(trd,i);

export(s,g,trd)$aggexport(s,trd)
	 = export(s,g,trd) * sexprt(s,trd) / aggexport(s,trd);

export(i,g,trd)$aggexport(i,trd)
	 = export(i,g,trd) * iexprt(i,trd) / aggexport(i,trd);

*	==========================================================
*	1. Illustrate identities implicit in the IMPLAN statistics.

parameter market	Market clearance check for IMPLAN (not rounded);

market("absolute",g) = 

*	Commodity supplies from production sectors and institutions:

		sum(s, make(s,g)) + sum(i, imake(i,g)) 

*	Commodity demands in production and institutions:

	    - sum(s, use(g,s)) - sum(i,  iuse(g,i));

*	Report the same market balance condition as a percentage
*	of aggregate supply:

market("percent",g)$( sum(s, make(s,g)) + sum(i, imake(i,g)) )
	 = 100 * (
	      sum(s, make(s,g)) + sum(i, imake(i,g)) 
	    - sum(s,  use(g,s)) - sum(i,  iuse(g,i)) ) /
	    ( sum(s, make(s,g)) + sum(i, imake(i,g)) );
option market:3:1:1;
display market;

PARAMETER       BALANCE(*,*)    Account balance check (round 5);


*	Supply to the domestic market (MAKE) plus exports (SEXPRT)
*	should equal the value of intermediate demand, factor demand
*	and imports:

BALANCE(S,"PROFIT") = 
	ROUND(    SUM(G, MAKE(S,G)   - USE(G,S))
		+ SUM(TRD, SEXPRT(S,TRD) - SIMPRT(TRD,S))
		- SUM(F, FD(F,S)), 5);

*	Market clearance check, rounded to five decimals:

BALANCE(G,"GMARKET") = ROUND( 
	          SUM(S,MAKE(S,G)   - USE(G,S)) 
		+ SUM(I, IMAKE(I,G) - IUSE(G,I)), 5);

*	Total supply and demand in national and international markets:

BALANCE(TRD,"TMARKET") = ROUND(  SUM(S, SEXPRT(S,trd)  - SIMPRT(TRD,S))
			     + SUM(F, FEXPRT(F,trd)  - FIMPRT(TRD,F))
			     + SUM(I, IEXPRT(I,trd)  - IIMPRT(TRD,I))
			     + SUM(TTRD,TRNSHP(TTRD,trd) - TRNSHP(TRD,TTrd)), 5);

*	Factor supplies (domestic and imports) less factor demands
*	(producers and institutions):

BALANCE(F,"FMARKET") = ROUND(  SUM(I,FS(I,F)) + SUM(TRD,FIMPRT(TRD,F))
		               - SUM(S,FD(F,S)) - SUM(TRD,FEXPRT(F,trd)), 5);

*	Income balance for institutions(goods and factor supply,
*	plus net transfer receipts, plus export earnings must equal
*	domestic commodity use plus imports:

BALANCE(I,"INCOME") = ROUND( SUM(G, IMAKE(I,G)) + SUM(F, FS(I,F))
                        + SUM(II, TRNSFER(I,II)-TRNSFER(II,I)) 
			+ SUM(TRD,  IEXPRT(I,trd)  -IIMPRT(TRD,I))
                        - SUM(G, IUSE(G,I)), 5);

*	Trade balance for national or international markets in a give
*	region equals sectoral exports, factor exports, and
*	institutional exports, plus net transhipments must equals
*	sectoral imports, factor imports and institutional imports:

BALANCE(TRD,"TRADE")  = ROUND( SUM(S,SEXPRT(S,trd)) + SUM(F,FEXPRT(F,trd))
			     + SUM(I, IEXPRT(I,trd)) 
			     + SUM(TTRD,(TRNSHP(TTRD,trd)-TRNSHP(TRD,TTrd)))
			     - SUM(S, SIMPRT(TRD,S)) - SUM(F, FIMPRT(TRD,F))
			     - SUM(I, IIMPRT(TRD,I)), 5);
DISPLAY BALANCE;

parameter trnsfr(*,i) Net transfers by institution and market;
trnsfr(TRD,i) = iimprt(TRD,i) - iexprt(i,trd);
trnsfr("total",i) = sum(TRD, trnsfr(TRD,i));
display trnsfr;

*	Although institutions may not balance trade in the domestic
*	or international markets individually, the economy
*	should have a net trade balance with the world as a whole:

PARAMETER ABALANCE  Asset balance;
ABALANCE(Trd)  = ROUND( SUM((G,S), EXPORT(S,G,trd)) + SUM(F,FEXPRT(F,trd))
	     +  SUM((G,I), EXPORT(I,G,trd)) + SUM(TTRD, TRNSHP(TTRD,trd))
	     - SUM(I, TRNSFR(TRD,i))
	     - SUM((G,S), IMPORT(trd,G,S))  - SUM(F, FIMPRT(TRD,F))
	     - SUM((G,I), IMPORT(trd,G,I))  - SUM(TTRD, TRNSHP(TRD,TTrd)), 5);

ABALANCE("TOTAL")  = SUM(TRD, ABALANCE(Trd));
DISPLAY ABALANCE;

*	==========================================================
*	2. Convert production struction into a commodity rather than
*	an industry basis.  

parameter supply, thetas;
$onuni
supply(s,"S") = make(s,s) + sum(TRD, export(s,s,trd));
supply(s,"Other") = sum(ss$(not sameas(ss,s)), make(ss,s) + 
				sum(TRD, export(ss,s,trd)));
supply(g,"I") = sum(i, imake(i,g) + sum(TRD, export(i,g,trd)));
supply(g,"total") = sum(s, make(s,g) + sum(TRD, export(s,g,trd))) +
		    sum(i, imake(i,g) + sum(TRD, export(i,g,trd)));

supply(s,"S")$supply(s,"total") = supply(s,"S") / supply(s,"total");
supply(s,"Other")$supply(s,"total") = supply(s,"Other") / supply(s,"total");
supply(s,"I")$supply(s,"total") = supply(s,"I") / supply(s,"total");
display supply;
$offuni

parameter	totalsupply(s)	Total sectoral output;
totalsupply(s) = SUM(GG, MAKE(S,GG)+SUM(TRD, EXPORT(S,GG,TRD)));

*	Fraction of production in sector s devoted to 
*	commodity g:

thetas(s,g)$totalsupply(s)
	= (MAKE(S,G) + SUM(TRD, EXPORT(S,G,TRD))) / totalsupply(s);

parameter	d0(s)		Sectoral output for domestic markets
		x0(s,trd)	Sectoral output for export
		y0(s)		Aggregate sectoral output
		id0(g,s)	Intermediate demand
		fd0(f,s)	Factor demand
		a0(g)		Aggregate supply
		m0(g,trd)	Foreign imports
		fe0(i,f)	Factor endowments
		deficit(i)	Net deficit
		c0(i,g)		Consumption demand;

*	Domestic output:

d0(g) = sum(s, make(s,g)) + sum(i, imake(i,g));
x0(g,trd) = sum(s, export(s,g,trd)) + sum(i, export(i,g,trd));
y0(g) = d0(g) + sum(trd, x0(g,trd));

*	Intermediate demand sums domestic and imported demand 
*	over all sectors in proportion to their production of 
*	commodity s:

id0(g,s) = sum(ss, thetas(ss,s) * (use(g,ss) + sum(trd, import(trd,g,ss))));

*	Factor demand:

fd0(f,g) = sum(s, thetas(s,g) * fd(f,s));

*	Treat institutional make as proprietary income:

fd0("prop",g) = fd0("prop",g) 
		+ sum(i, imake(i,g))
		+ sum((i,trd), export(i,g,trd));

*	Factor endowments and stock change:

fe0(i,f) = fs(i,f);

*	Include proprietary income for institutions 
*	representing institution make and export:

fe0(i,"prop") = fe0(i,"prop") + sum(g, imake(i,g)) + sum((g,trd), export(i,g,trd));
fe0("stk",f) = fe0("stk",f) + sum(TRD, fimprt(TRD,f)-fexprt(f,TRD));

*	Exports to international markets (all by sectors):


*	Imports from international markets are accounted for in
*	both sectoral and institutional accounts:

m0(g,trd) = sum(s, import(trd,g,s)) + sum(i, import(trd,g,i));

*	Consumption by commodity and institution equals use by
*	commodity and institution less output, plus imports 
*	minus exports to domestic markets:

c0(i,g) = iuse(g,i) + sum(TRD, import(trd,g,i));

*	Verify that these definitions satisfy some Walrasian 
*	identities for profiTRD, market cleance and aggregate income
*	balance:

parameter	profit(s)	Production profit
		armmkt(s)	Armington market check
		facmkt(*)	Factor market
		closure		Aggregate closure;

*	Domestic (intra-national) trade flows:

*	Aggregate supply then equals intranational supply
*	plus imports from foreign sources:

a0(g) = sum(trd,m0(g,trd)) + d0(g);

*	Zero profit implies that total domestic and export
*	sales equals the total of intermedaite and factor inputs:

profit(s) = d0(s) + sum(trd,x0(s,trd)) - sum(g, id0(g,s)) - sum(f, fd0(f,s));

*	Armington market clearance implies that aggregate
*	supply equals institutional and intermediate demand:

armmkt(g) = a0(g)  - sum(i, c0(i,g)) - sum(s, id0(g,s));

*	Factor markets:

facmkt(f) = sum(s, fd0(f,s)) - sum(i, fe0(i,f));

*	Trade deficit with rest-of-world:

deficit(i) = sum(g, c0(i,g)) - sum(f, fe0(i,f));

*	Closure implies that net income imbalances in
*	all regions equal the trade deficit:

closure = sum(i, deficit(i)) - sum((g,trd), m0(g,trd)-x0(g,trd));

*	Produce a consistency report of the base year identities
*	as captured in the IMPLAN data:

display  profit, armmkt, facmkt, closure, deficit;

option id0:3:1:1;
option fd0:3:1:1;
option fe0:3:1:1;
option c0:3:1:1;

parameter	y0(s)	Base year output,
		d0(s)	Domestic supply,
		e0(s)	Exports
		a0(s)   Aggregate supply including imports;

e0(s) = sum(trd,x0(s,trd));
d0(s) = sum(g, id0(g,s)) + sum(f, fd0(f,s)) - e0(s);
y0(s) = sum(g, id0(g,s)) + sum(f, fd0(f,s));
a0(g) = d0(g) + sum(trd,m0(g,trd));

parameter	
	kd0(*)		Capital return
	bt0(s)		Business taxes
	fica		FICA tax revenue
	pld0		Reference wage rate (gross of tax)
	ld0(s)		Labor demand

	i0(g)		Investment demand by commodity
	inv0		Aggregate investment
	invd0		Investment demand by household

	g0(g,pub)	Public demand by commodity
	gov0(pub)	Aggregate public demand

	cd0(g,hh)	Consumer demand

	le0(hh)		Labor endowment
	ke0(hh)		Capital endowment

	incadj(hh)	Base year net transfer ;


*	Round away small numbers:

x0(s,trd) = max(0, round(x0(s,trd), 4));
id0(g,s)  = max(0, round(id0(g,s),  4));
d0(g)	  = max(0, round(d0(g),	    4));
m0(g,trd) = max(0, round(m0(g,trd), 4));
fe0(i,f)  = round(fe0(i,f),  4);
c0(i,g)   = round(c0(i,g),   4);
fd0(f,s)  = round(fd0(f,s),  4);

y0(s) = d0(s) + sum(trd, x0(s,trd));
a0(s) = d0(s) + sum(trd, m0(s,trd));

bt0(s) = fd0("btax",s);
kd0(s)	= fd0("prop",s) + fd0("othp",s);

fica = sum(pub,fe0(pub,"empl"));
display fica;

g0(g,pub) = c0(pub,g);
cd0(g,hh) = c0(hh,g);

*	Impute investment demand to households:

i0(g) = a0(g)  - sum(hh, cd0(g,hh)) - sum(pub,g0(g,pub)) - sum(s, id0(g,s));

*	Drop products which are not in the dataset:

loop(g$(not a0(g)),
  i0(g) = round(  i0(g), 3);
  cd0(g,hh) = round(  cd0(g,hh), 3);
  g0(g,pub) = round(  g0(g,pub), 3);
  id0(g,s) = round(  id0(g,s), 3);
);

gov0(pub)	= sum(g, g0(g,pub));

*	Impute wage payments, capital payments or taxes:

ld0(s) = max(d0(s)+sum(trd,x0(s,trd)) - kd0(s) - sum(g, id0(g,s)) - bt0(s), 0);
kd0(s) = max(d0(s)+sum(trd,x0(s,trd)) - ld0(s) - sum(g, id0(g,s)) - bt0(s), 0);
bt0(s) = d0(s)+sum(trd,x0(s,trd)) - kd0(s) - sum(g, id0(g,s)) - ld0(s);

*	Scale labor endowments to match imputed wage payments:

le0(hh)	= fe0(hh,"empl")/(fica+sum(h_h, fe0(h_h,"empl"))) * sum(s, ld0(s));
fica = fica/(fica+sum(h_h, fe0(h_h,"empl"))) * sum(s, ld0(s));

ke0(hh)	= (fe0(hh,"prop") + fe0(hh,"othp"))/
	  sum(h_h,fe0(h_h,"prop") + fe0(h_h,"othp")) * sum(s, kd0(s));


inv0  = sum(g, i0(g));

invd0(hh)$sum(h_h, max(0, trnsfer("inv",h_h) - trnsfer(h_h,"inv")))
	= max(0, trnsfer("inv",hh) - trnsfer(hh,"inv")) /
	    sum(h_h, max(0, trnsfer("inv",h_h) - trnsfer(h_h,"inv"))) * inv0;

incadj(hh) = sum(g, cd0(g,hh)) + invd0(hh) - ke0(hh) - le0(hh);

closure = sum(hh, incadj(hh)) + sum(pub,gov0(pub)) - fica - sum(s,bt0(s)) 
		- sum((g,trd), m0(g,trd)-x0(g,trd));
display closure;

*	==========================================================
*	3. Specify an MPSGE model based on these statistics and
*	verify that the resulting data represent an equilibrium
*	for this model.  This is an utterly simplistic model in
*	which we treat all institutions symmetrically.

$ontext
$model:soe

$sectors:
	y(s)$y0(s)	! Sectoral production
	a(s)$a0(s)	! Armington aggregation
	gov(pub)	! Public output
	inv		! Investment

$commodities:
	p(s)$d0(s)	! Sectoral output prices
	pa(s)$a0(s)	! Armington aggregate prices
	pinv		! New investment
	pgov(pub)	! Public output
	pl		! Wage rate
	rk		! Return to capital
	pfx		! Foreign exchange
	ptax		! Business taxes

$consumers:
	rh(hh)		! Insitutions
	govt(pub)	! Government (different levels)
	taxrev		! Tax revenue

$prod:y(s)$y0(s)  s:0 t:1  va:1
	o:p(s)	q:d0(s)
	o:pfx	q:(sum(trd,x0(s,trd)))
	i:pa(g)	q:id0(g,s)
	i:pl	q:ld0(s)	va:
	i:rk	q:kd0(s)	va:	
	i:ptax	q:(max(bt0(s),0))
	o:ptax	q:(max(-bt0(s),0))

$prod:a(g)$a0(g)  s:2  m:4
	o:pa(g)	q:a0(g)
	i:p(g)	q:d0(g)	
	i:pfx	q:m0(g,"ftrd")	m:
	i:pfx	q:m0(g,"dtrd")	m:

$prod:inv
	o:pinv	q:inv0
	i:pa(g)	q:(max(0,i0(g)))
	o:pa(g)	q:(max(0,-i0(g)))

$prod:gov(pub)
	o:pgov(pub)	q:gov0(pub)
	i:pa(g)		q:(max(g0(g,pub),0))
	o:pa(g)		q:(max(-g0(g,pub),0))

$demand:rh(hh) s:1
	e:rk		q:ke0(hh)
	e:pl		q:le0(hh)
	e:pfx		q:incadj(hh)
	e:pinv		q:(-invd0(hh))
	d:pa(g)		q:cd0(g,hh)

$demand:govt(pub)
	d:pgov(pub)	q:gov0(pub)
	e:pfx		q:gov0(pub)

$demand:taxrev
	e:pl		q:fica
	e:ptax		q:(sum(s,bt0(s)))
	d:pfx

$offtext
$sysinclude mpsgeset soe

option mcp=pathold;

soe.iterlim = 0;
*.$include soe.gen
*.solve soe using mcp;
execute_unload 'data\%ds%.gdx', x0,m0,id0,kd0,ld0,bt0,i0,g0,le0,ke0,cd0,invd0,fica;

