$title	MPSGE Model for Implan Data

$if not set ds  $set ds gtap

SET	r(*)	Regions (states),
	s(*)	Sectors and goods;
	
$gdxin data\%ds%
$load r s

alias (s,g);

SET trd  Trade partners in the IMPLAN data/
		FTRD "Foreign Trade (25001)"
		DTRD "Domestic Trade (28001)" /,

	hh	Private (household) institutions /
		HHL "Households LT5k   (10001)"
		HH5 "Households 5-10k  (10002)"
		HH10 "Households 10-15k (10003)"
		HH15 "Households 15-20k (10004)"
		HH20 "Households 20-30k (10005)"
		HH30 "Households 30-40k (10006)"
		HH40 "Households 40-50k (10007)"
		HH50 "Households 50-70k (10008)"
		HH70 "Households 70k+   (10009)"/,

	pub	Public (government) institutions /
		FND "Federal Government NonDefense (11001)"
		FDF "Federal Government Defense (11002)"
		FIN "Federal Government Investment (11003)"
		SLN "State Local Govt NonEducation (12001)"
		SLE "State Local Govt Education (12002)"
		SIN "State Local Govt Investment (12003)" /;

alias (h,hh);

parameter
	x0(r,s,trd)	Exports by market
	m0(r,g,trd)	Imports from domestic and foreign sources
	id0(r,g,s)	Intermediate demand
	kd0(r,s)	Capital return
	ld0(r,s)	Labor demand
	bt0(r,s)	Busines taxes
	fica(r)		Labor tax rate (social security)
	i0(r,g)		Investment demand by commodity
	g0(r,g,pub)	Public demand by commodity
	le0(r,hh)	Labor endowment
	ke0(r,hh)	Capital endowment
	cd0(r,g,hh)	Consumer demand
	invd0(r,hh)	Investment
	n0(g)		Intra-national trade;

$load x0 m0 id0 kd0 ld0 bt0 fica i0 g0 le0 ke0 cd0 invd0 

parameter d0, y0, a0, gov0, inv0, incadj;
d0(r,s) = round(bt0(r,s) + kd0(r,s) + ld0(r,s) + sum(g, id0(r,g,s)) - sum(trd,x0(r,s,trd)),6);
y0(r,s) = d0(r,s) + sum(trd, x0(r,s,trd));
a0(r,s) = sum(trd,m0(r,s,trd)) + d0(r,s);
gov0(r,pub) = sum(g, g0(r,g,pub));
inv0(r) = sum(g, i0(r,g));
incadj(r,hh) = sum(g, cd0(r,g,hh)) + invd0(r,hh) - ke0(r,hh) - le0(r,hh);
n0(g) = sum(r, x0(r,g,"dtrd"));

*	Initially set sector-specific capital to zero:

parameter	
	tc(r,s)		Carbon tax rate
	lm0(r,s)	Mobile labor
	ks0(r,s)	Sector-specific capital
	kes0(r,s,hh)	Sector-specific capital endowments
	carbontarget	Carbon emissions target /0/
	emit(r,s)	Base year emissions
	pw(s)		World market price;

emit(r,s) = 0;
lm0(r,s) = 0;
tc(r,s) = 0;
ks0(r,s) = 0;
kes0(r,s,hh) = 0;
pw(s) = 1;


*	Specify an MPSGE model based on these statistics and
*	verify that the resulting data represent an equilibrium
*	for this model.  This is an utterly simplistic model in
*	which we treat all institutions symmetrically.

$ontext

$model:soe

$sectors:
	y(r,s)$y0(r,s)	! Sectoral production
	a(r,s)$a0(r,s)	! Armington aggregation
	gov(r,pub)	! Public output
	inv(r)		! Investment

$commodities:
	p(r,s)$d0(r,s)	! Sectoral output prices
	pa(r,s)$a0(r,s)	! Armington aggregate prices
	pn(s)$n0(s)	! Intra-national trade price 
	pinv(r)		! New investment
	pgov(r,pub)	! Public output
	pl(r,s)$ld0(r,s) ! Wage rate
	plm(r)$sum(s,lm0(r,s))	! Wage rate (mobile labor)
	rk(r)$(sum(s,kd0(r,s)))		! Return to capital
	rks(r,s)$ks0(r,s)		! Return to sector-specific capital
	pfx		! Foreign exchange
	ptax(r)		! Business taxes
	pcarb$carbontarget

$consumers:
	rh(r,hh)	! Insitutions
	govt(r,pub)	! Government (different levels)
	taxrev(r)	! Tax revenue

$auxiliary:
	unempl(r,s)$(ld0(r,s) and unempl.up(r,s))
	target$carbontarget

$prod:y(r,s)$y0(r,s)  s:0 t:1  va:1
	o:p(r,s)	q:d0(r,s)	 a:taxrev(r)	t:tc(r,s)
	o:pfx		q:x0(r,s,"ftrd") a:taxrev(r)	t:tc(r,s)
	o:pn(s)		q:x0(r,s,"dtrd") a:taxrev(r)	t:tc(r,s)
	i:pcarb		q:emit(r,s)
	i:pa(r,g)	q:id0(r,g,s)
	i:pl(r,s)	q:ld0(r,s)	va:
	i:plm(r)	q:lm0(r,s)	va:
	i:rk(r)		q:kd0(r,s)	va:	
	i:rks(r,s)	q:ks0(r,s)	va:
	i:ptax(r)	q:(max(bt0(r,s),0))
	o:ptax(r)	q:(max(-bt0(r,s),0))

$prod:a(r,g)$a0(r,g)  s:4  m:8
	o:pa(r,g)	q:a0(r,g)
	i:p(r,g)	q:d0(r,g)	
	i:pfx		q:(pw(g)*m0(r,g,"ftrd"))	m:	p:(1/pw(g))
	i:pn(g)		q:m0(r,g,"dtrd")		m:

$prod:inv(r)
	o:pinv(r)	q:inv0(r)
	i:pa(r,g)	q:(max(0,i0(r,g)))
	o:pa(r,g)	q:(max(0,-i0(r,g)))

$prod:gov(r,pub)
	o:pgov(r,pub)	q:gov0(r,pub)
	i:pa(r,g)	q:(max(g0(r,g,pub),0))
	o:pa(r,g)	q:(max(-g0(r,g,pub),0))

$demand:rh(r,hh) s:1
	e:rk(r)		q:ke0(r,hh)
	e:rks(r,s)	q:kes0(r,s,hh)
	e:pl(r,s)$ld0(r,s)	q:(le0(r,hh)*ld0(r,s)/sum(g,ld0(r,g)))
	e:plm(r)	q:(sum(s,lm0(r,s)))
	e:pl(r,s)$unempl.up(r,s)	q:(-le0(r,hh)*ld0(r,s)/sum(g,ld0(r,g)))	   r:unempl(r,s)
	e:pfx		q:incadj(r,hh)
	e:pinv(r)	q:(-invd0(r,hh))
	d:pa(r,g)	q:cd0(r,g,hh)

$demand:govt(r,pub)
	d:pgov(r,pub)	q:gov0(r,pub)
	e:pcarb		q:carbontarget	r:target
	e:pfx		q:gov0(r,pub)

$constraint:target$carbontarget
	pfx * 25/1e6 =e= pcarb;

$demand:taxrev(r)
	e:pl(r,s)$ld0(r,s)		q:(fica(r)*ld0(r,s)/sum(g,ld0(r,g)))
	e:plm(r)$sum(s,lm0(r,s))	q:fica(r)
	e:pl(r,s)$unempl.up(r,s)	q:(-fica(r)*ld0(r,s)/sum(g,ld0(r,g)))	r:unempl(r,s)
	e:ptax(r)			q:(sum(s,bt0(r,s)))
	d:pfx

$constraint:unempl(r,s)$(ld0(r,s) and unempl.up(r,s))
	pl(r,s) =g= pfx;

$offtext
$sysinclude mpsgeset soe

$if "%1"=="nosolve" $exit

soe.workspace = 40;
soe.iterlim = 0;
$include soe.gen
solve soe using mcp;
