$title	Adjustment program to produce a dataset in which domestic trade flows balance

$set ds 528

$batinclude model nosolve

parameter	mchk	Cross check of import markets, 
		pfxchk	Cross check of foreign exchange;

mchk(s)$sum(r,x0(r,s,"dtrd")) = sum(r,m0(r,s,"dtrd"))/sum(r,x0(r,s,"dtrd"));

*	When mchk(s) < 1, then domestic exports exceed domestic imports.
*	The solution is to proportionally reduce domestic exports and 
*	increase international exports, holding aggregate exports constant.

loop(s$(0 < mchk(s)  and mchk(s) < 1),
	x0(r,s,"dtrd") = mchk(s)*x0(r,s,"dtrd");
	x0(r,s,"ftrd") = x0(r,s,"ftrd") + x0(r,s,"dtrd") * (1/mchk(s) - 1);
);

*	When mchk(s) > 1, then domestic imports exceed domestic exports.
*	We then proportionally reduce domestic imports and increase
*	international imports, so that aggregate imports remain constant.

m0(r,s,"dtrd")$(mchk(s) gt 1) = m0(r,s,"dtrd")/mchk(s);
m0(r,s,"ftrd")$(mchk(s) gt 1) = m0(r,s,"ftrd") + m0(r,s,"dtrd") * (mchk(s) - 1);
	

set ftrd(trd) /ftrd/;
pfxchk = sum((r,s,ftrd), x0(r,s,ftrd) - m0(r,s,ftrd)) + sum((r,hh), incadj(r,hh)) + sum((r,pub), gov0(r,pub))
		- sum((r,s), fica(r) * ld0(r,s)/sum(g,ld0(r,g))) - sum((r,s), bt0(r,s));
display pfxchk;

parameter mktchk;
mktchk(s,trd) = sum(r, x0(r,s,trd) - m0(r,s,trd));
display mktchk;

abort$(smax(s, abs(mktchk(s,"dtrd"))) gt 1.e-5) "Error -- domestic markets are not in balance";

execute_unload 'data\implan',r,s,x0,m0,id0,kd0,ld0,bt0,fica,i0,g0,le0,ke0,cd0,invd0;

